import Helper from './helper'
import Registry from '../registry'
import { DSL, ActionDefinition } from './definitions/action'
import { dotpath } from '../util'

class Action extends Helper {
  static registryInterface (registry) {
    return {
      actionsForCLI: {
        get () {
          return registry.query(action => action.get('definition.interfaces.cli'))
        }
      }
    }
  }

  /**
   * Configures a CLI utility such as commander to be able to expose
   * this action over a CLI interface.  Any additional context passed
   * from the CLI will be included, and we will make sure the project
   * is set
  */
  attachCLI (commander, dispatchFn, context = {}) {
    context.project = context.project || this.project

    if(this.get('definition.interfaces.cli')) {
      this.interfaces.cli.call(this, commander, dispatchFn)
    }
  }

  get interfaces() {
    return this.definition.interfaces
  }

  get helperClass () {
    return Action
  }

  get definitionClass () {
     return ActionDefinition
  }
}

Action.DSL = DSL
Action.Definition = ActionDefinition
Action.apiMethods = ActionDefinition.apiMethods

module.exports = Action

