let requireSubvert = require('require-subvert')(__dirname)
import { stub, spy } from 'sinon'

describe('logger', () => {

  let project
  let options
  let winston
  let logger

  before(() => {
    winston = {
      Logger: () => {},
      transports: {
        File: () => {},
        Console: () => {}
      }
    }
    requireSubvert.subvert('winston', winston)
    logger = requireSubvert.require('../../src/logger').default
  })

  after(() => {
    requireSubvert.cleanUp()
  })

  beforeEach(() => {
    project = {
      ensurePath: () => {},
      path: () => {}
    }
    options = {}
  })

  afterEach(() => {
  })

  it('should do nothing if the project already has a logger', () => {
    project.logger = { name: 'this is a test' }
    let output = logger(project, options)
    output.should.equal(project.logger)
  })

  it('should ensure log folder exists', () => {
    let ensurePath = stub(project, 'ensurePath')
    logger(project, options)
    ensurePath.should.be.calledWith('logs')
    ensurePath.restore()
  })

  it('should create a file transport', () => {
    let file = stub(winston.transports, 'File')
    logger(project, options)
    file.should.be.calledWithNew
    file.restore()
  })

  it('should create a console transport', () => {
    let cons = stub(winston.transports, 'Console')
    logger(project, options)
    cons.should.be.calledWithNew
    cons.restore()
  })

  it('should create logging convenience methods', () => {
    let log = spy()
    let mock = {
      log: log
    }
    let loggerStub = stub(winston, 'Logger').returns(mock)

    logger(project, options)

    project.debug('test0')
    log.should.be.calledWith('debug', 'test0')

    project.log('test1')
    log.should.be.calledWith('info', 'test1')

    project.info('test2')
    log.should.be.calledWith('info', 'test2')

    project.warn('test3')
    log.should.be.calledWith('warn', 'test3')

    project.error('test4')
    log.should.be.calledWith('error', 'test4')
    
    loggerStub.restore()
  })
})
